/*
 * Tpg300Channel.cpp
 *
 *  Created on: Jun 12, 2012
 *      Author: mdm
 */

#include "Tpg300Channel.h"
#include "Tpg300Proxy.h"
#include "Tpg300Utils.h"
#include "ThreadGuard.h"

namespace Tpg300_ns
{

//--------------------------------------------------------
// Tpg300Channel::Tpg300Channel()
//--------------------------------------------------------
Tpg300Channel::Tpg300Channel(Tpg300Proxy* p, Channel c) :
		proxy(p), channel(c)
{
    pressure = 0;
    state = Tango::UNKNOWN;
    status = "Unknown status";
    name = "Unknown channel name";
}

//--------------------------------------------------------
// Tpg300Channel::update_channel()
//--------------------------------------------------------
void Tpg300Channel::update()
{
	string result;

	try
	{
		switch(channel)
		{
		case CHANNEL_0:
			result = proxy->write_read("PA1");
			break;
		case CHANNEL_1:
			result = proxy->write_read("PA2");
			break;
		case CHANNEL_2:
			result = proxy->write_read("PB1");
			break;
		case CHANNEL_3:
			result = proxy->write_read("PB2");
			break;
		}

		ThreadGuard lock(mutex);

		pressure = parse_pressure(result);
		state = parse_state(result);
		status = parse_status(result);
	}
	catch(Tango::DevFailed &ex)
	{
		state = Tango::FAULT;
		status = Tpg300Utils::get_trace(ex);
	}
}

//--------------------------------------------------------
// Tpg300Channel::get_pressure()
//--------------------------------------------------------
Tango::DevDouble Tpg300Channel::get_pressure()
{
	ThreadGuard lock(mutex);

    return pressure;
}

//--------------------------------------------------------
// Tpg300Channel::get_state()
//--------------------------------------------------------
Tango::DevState Tpg300Channel::get_state()
{
	ThreadGuard lock(mutex);

    return state;
}

//--------------------------------------------------------
// Tpg300Channel::get_status()
//--------------------------------------------------------
string Tpg300Channel::get_status()
{
	ThreadGuard lock(mutex);

    return status;
}

//--------------------------------------------------------
// Tpg300Channel::set_name()
//--------------------------------------------------------
void Tpg300Channel::set_name(string n)
{
	ThreadGuard lock(mutex);

	name = n;
}

//--------------------------------------------------------
// Tpg300Channel::get_name()
//--------------------------------------------------------
string Tpg300Channel::get_name()
{
	ThreadGuard lock(mutex);

	return name;
}

//--------------------------------------------------------
// Tpg300Channel::on()
//--------------------------------------------------------
void Tpg300Channel::on()
{
	read_write_state(channel, ON);
}

//--------------------------------------------------------
// Tpg300Channel::off()
//--------------------------------------------------------
void Tpg300Channel::off()
{
	read_write_state(channel, OFF);
}

//========================================================
//	Utilities methods
//========================================================

//--------------------------------------------------------
// Tpg300Channel::read_write_state()
//--------------------------------------------------------
void Tpg300Channel::read_write_state(Channel channel, ChannelState new_state)
{
	string result = proxy->write_read("SEN,0,0,0,0");

	if(result.size() == 7)
	{
		int channels_states[4] = {0};

		channels_states[0] = atoi( &result.at(0) );
		channels_states[1] = atoi( &result.at(2) );
		channels_states[2] = atoi( &result.at(4) );
		channels_states[3] = atoi( &result.at(6) );

		channels_states[channel] = new_state;

		stringstream command_str;
		command_str << "SEN,"
				<< channels_states[0] << ","
				<< channels_states[1] << ","
				<< channels_states[2] << ","
				<< channels_states[3];

		proxy->write_read(command_str.str());
	}
	else
		Tango::Except::throw_exception("", "Parsing error", "read_write_state");

}

//--------------------------------------------------------
// Tpg300Channel::parse_pressure()
//--------------------------------------------------------
Tango::DevDouble Tpg300Channel::parse_pressure(string result)
{
	Tango::DevDouble pressure = 0;

    size_t pos;
    if((pos = result.find(POINT)) != string::npos)
    {
    	pressure = strtod(result.substr(pos+1, result.size()).c_str(), 0);
    }
    else
    	Tango::Except::throw_exception("", "Parsing error", "parse_pressure");

    return pressure;
}

//--------------------------------------------------------
// Tpg300Channel::parse_state()
//--------------------------------------------------------
Tango::DevState Tpg300Channel::parse_state(string result)
{
	Tango::DevState state = Tango::FAULT;

    size_t pos;
    if( (pos = result.find(POINT)) != string::npos)
    {
    	switch(atoi(result.substr(0, pos).c_str()))
    	{
    	case 0:
    		state = Tango::ON;
    		break;
    	case 1:
    		state = Tango::ALARM;
    		break;
    	case 2:
    		state = Tango::ALARM;
    		break;
    	case 3:
    		state = Tango::ALARM;
    		break;
    	case 4:
    		state = Tango::OFF;
    		break;
    	case 5:
    		state = Tango::DISABLE;
    		break;
    	}
    }
    else
    	Tango::Except::throw_exception("", "Parsing error", "parse_state");

    return state;
}

//--------------------------------------------------------
// Tpg300Channel::parse_status()
//--------------------------------------------------------
string Tpg300Channel::parse_status(string result)
{
	string status = "Parsing error";

    size_t pos;
    if( (pos = result.find(POINT)) != string::npos)
    {
    	switch(atoi(result.substr(0, pos).c_str()))
    	{
    	case 0:
    		status = "Measurement data OK";
    		break;
    	case 1:
    		status = "Under range";
    		break;
    	case 2:
    		status = "Over range";
    		break;
    	case 3:
    		status = "Measuring circuit error";
    		break;
    	case 4:
    		status = "Measuring circuit switched off";
    		break;
    	case 5:
    		status = "No hardware";
    		break;
    	}
    }
    else
    	Tango::Except::throw_exception("", "Parsing error", "parse_status");

    return status;
}

}
