/*
 * Tpg300Error.cpp
 *
 *  Created on: Jul 17, 2012
 *      Author: mdm
 */

#include <Tpg300Error.h>
#include <Tpg300Proxy.h>
#include <Tpg300Task.h>
#include <Tpg300Utils.h>
#include <ThreadGuard.h>

namespace Tpg300_ns
{

//--------------------------------------------------------
// Tpg300Error::Tpg300Error()
//--------------------------------------------------------
Tpg300Error::Tpg300Error(Tpg300Proxy* p) : proxy(p)
{
    state = Tango::UNKNOWN;
    status = "Unknown status";
}

//--------------------------------------------------------
// Tpg300Error::update()
//--------------------------------------------------------
void Tpg300Error::update()
{
	ThreadGuard lock(mutex);

	try
	{
		string result = proxy->write_read("ERR");

		state = parse_state(result);
		status = parse_status(result);
	}
	catch(Tango::DevFailed &ex)
	{
		state = Tango::FAULT;
		status = Tpg300Utils::get_trace(ex);
	}
}

//--------------------------------------------------------
// Tpg300Error::get_state()
//--------------------------------------------------------
Tango::DevState Tpg300Error::get_state()
{
	ThreadGuard lock(mutex);

	return state;
}

//--------------------------------------------------------
// Tpg300Error::get_status()
//--------------------------------------------------------
std::string Tpg300Error::get_status()
{
	ThreadGuard lock(mutex);

	return status;
}

//========================================================
//	Utilities methods
//========================================================

//--------------------------------------------------------
// Tpg300Error::parse_state()
//--------------------------------------------------------
Tango::DevState Tpg300Error::parse_state(std::string result)
{
	Tango::DevState state;

	switch(atoi(result.c_str()))
	{
	case 0:
		state = Tango::STANDBY;
		break;
	case 10:
		state = Tango::ALARM;
		break;
	case 100:
		state = Tango::ALARM;
		break;
	case 1000:
		state = Tango::ALARM;
		break;
	default:
    	Tango::Except::throw_exception("", "Parsing error", "parse_state");
		break;
	}

    return state;
}

//--------------------------------------------------------
// Tpg300Error::parse_status()
//--------------------------------------------------------
std::string Tpg300Error::parse_status(std::string result)
{
	string status;

	switch(atoi(result.c_str()))
	{
	case 0:
		status = "No error";
		break;
	case 10:
		status = "Invalid parameter";
		break;
	case 100:
		status = "Hardware not installed";
		break;
	case 1000:
		status = "Tpg error";
		break;
	default:
    	Tango::Except::throw_exception("", "Parsing error", "parse_status");
		break;
	}

	return status;
}

}


