/*
 * Tpg300Proxy.cpp
 *
 *  Created on: May 22, 2012
 *      Author: mdm
 */

#include <iomanip>
#include "Tpg300Proxy.h"
#include "ThreadGuard.h"

namespace Tpg300_ns
{

//--------------------------------------------------------
// Tpg300Proxy::Tpg300Proxy()
//--------------------------------------------------------
Tpg300Proxy::Tpg300Proxy(string serial_device, Tango::DevULong timeout,
		Tango::DevULong proxySleep)
{
	proxy = new Tango::DeviceProxy(serial_device);
	proxy->set_timeout_millis(timeout);

	secs = proxySleep / 1000;
	nanosecs = (proxySleep % 1000) * 1000000;
}

//--------------------------------------------------------
// Tpg300Proxy::~Tpg300Proxy()
//--------------------------------------------------------
Tpg300Proxy::~Tpg300Proxy()
{
	delete proxy;
}

//--------------------------------------------------------
// Tpg300Proxy::write_read()
//--------------------------------------------------------
std::string Tpg300Proxy::write_read(std::string input_str)
{
	ThreadGuard lock(mutex);

    flush_data();

	string output_str = serial_in_out(input_str);

    if(output_str.find(NAK) != string::npos)
    	Tango::Except::throw_exception("","Negative acknowledge received","write_read");

    omni_thread::sleep(secs,nanosecs);

    input_str = ENQ;

    output_str = serial_in_out(input_str);

    return output_str;
}

//========================================================
//	Utilities methods
//========================================================

//--------------------------------------------------------
// Tpg300Proxy::flush_data()
//--------------------------------------------------------
void Tpg300Proxy::flush_data()
{
	Tango::DeviceData dev_data;
	Tango::DevLong flush = 2;

	dev_data << flush;
    proxy->command_inout("DevSerFlush", dev_data);
}

//--------------------------------------------------------
// Tpg300Proxy::serial_in_out()
//--------------------------------------------------------
string Tpg300Proxy::serial_in_out(std::string command_str)
{
	Tango::DeviceData dev_data;

	command_str.push_back(CR);

#ifdef _VERBOSE_DEBUG
    print_request_debug(command_str);
#endif

    dev_data << command_str;
    proxy->command_inout("DevSerWriteString", dev_data);

	string output_str;

    dev_data = proxy->command_inout("DevSerReadLine");
    dev_data >> output_str;

#ifdef _VERBOSE_DEBUG
    print_response_debug(output_str);
#endif

	size_t pos;

	if ((pos=output_str.find_first_of(LF)) != string::npos)
		output_str = output_str.substr(pos+1, string::npos);

	if ((pos=output_str.find_last_of(CR)) != string::npos)
		output_str = output_str.substr(0, pos);

    return output_str;
}

//--------------------------------------------------------
// Tpg300Proxy::print_request_debug()
//--------------------------------------------------------
void Tpg300Proxy::print_request_debug(string command_str)
{
	stringstream debug_stream;

	debug_stream << ">>>>>>>>>>>>>>>>>>>>>>>>>>>" << endl;
	for(unsigned int j=0; j<command_str.size(); j++)
	{
		debug_stream << "0x" << internal << setw(2) << setfill('0')
				<< hex << int(command_str[j]) << " | ";

		if(int(command_str[j]) > 31)
			debug_stream << command_str[j];

		debug_stream << endl;
	}
	debug_stream << ">>>>>>>>>>>>>>>>>>>>>>>>>>>" << endl;

	cout << debug_stream.str() << endl;
}

//--------------------------------------------------------
// Tpg300Proxy::print_response_debug()
//--------------------------------------------------------
void Tpg300Proxy::print_response_debug(string command_str)
{
	stringstream debug_stream;

	debug_stream << "<<<<<<<<<<<<<<<<<<<<<<<<<<<" << endl;
	for(unsigned int j=0; j<command_str.size(); j++)
	{
		debug_stream << "0x" << internal << setw(2)
				<< setfill('0') << hex << int(command_str[j]) << " | ";

		if(int(command_str[j]) > 31)
			debug_stream << command_str[j];

		debug_stream << endl;
	}
	debug_stream << "<<<<<<<<<<<<<<<<<<<<<<<<<<<" << endl;

	cout << debug_stream.str() << endl;
}

}
