/*
 * Tpg300SetPoint.cpp
 *
 *  Created on: Jun 12, 2012
 *      Author: mdm
 */

#include <cmath>
#include "Tpg300SPManager.h"
#include "Tpg300Utils.h"
#include "Tpg300Proxy.h"
#include "ThreadGuard.h"

namespace Tpg300_ns
{

//--------------------------------------------------------
// Tpg300SetPoint::Tpg300SetPoint()
//--------------------------------------------------------
Tpg300SPManager::Tpg300SPManager(Tpg300Proxy* p) :
		proxy(p)
{
    valid = false;
    trace = "Not yet initialized";
}

//--------------------------------------------------------
// Tpg300SPManager::update_setpoints()
//--------------------------------------------------------
void Tpg300SPManager::update()
{
	ThreadGuard lock(mutex);

	try
	{
		string result = proxy->write_read("SPS");

		if(result.size() == 11)
		{
			setpoints[0] = atoi(&result.at(0));
			setpoints[1] = atoi(&result.at(2));
			setpoints[2] = atoi(&result.at(4));
			setpoints[3] = atoi(&result.at(6));
			setpoints[4] = atoi(&result.at(8));
			setpoints[5] = atoi(&result.at(10));

			valid = true;
			trace = "No exception";
		}
		else
		{
			valid = false;
			trace = "Parsing error";
		}
	}
	catch(Tango::DevFailed &ex)
	{
		valid = false;
		trace = Tpg300Utils::get_trace(ex);
	}
}

//--------------------------------------------------------
// Tpg300SPManager::get_setpoint_value()
//--------------------------------------------------------
void Tpg300SPManager::get_setpoint_value(Tango::DevBoolean* attr_SetPointStatus_read, long size)
{
	ThreadGuard lock(mutex);

	if(size != SETPOINT_SIZE)
		Tango::Except::throw_exception("", "Attribute wrong size", "get_setpoint_value");

	if(!valid)
		Tango::Except::throw_exception("", trace, "get_setpoint_value");

	for(unsigned int i=0; i<SETPOINT_SIZE; i++)
			attr_SetPointStatus_read[i] = setpoints[i];

}

//--------------------------------------------------------
// Tpg300SPManager::get_lower_threshold()
//--------------------------------------------------------
Tango::DevDouble Tpg300SPManager::get_lower_threshold(SetPoint setpoint)
{
	SetPointConf conf = get_setpoint_conf(setpoint);

	return conf.lower_threshold;
}

//--------------------------------------------------------
// Tpg300SPManager::set_lower_threshold()
//--------------------------------------------------------
void Tpg300SPManager::set_lower_threshold(SetPoint setpoint, Tango::DevDouble lower_threshold)
{
	SetPointConf conf = get_setpoint_conf(setpoint);

	conf.lower_threshold = lower_threshold;

	set_setpoint_conf(setpoint, conf);
}

//--------------------------------------------------------
// Tpg300SPManager::get_upper_threshold()
//--------------------------------------------------------
Tango::DevDouble Tpg300SPManager::get_upper_threshold(SetPoint setpoint)
{
	SetPointConf conf = get_setpoint_conf(setpoint);

	return conf.upper_threshold;
}

//--------------------------------------------------------
// Tpg300SPManager::set_upper_threshold()
//--------------------------------------------------------
void Tpg300SPManager::set_upper_threshold(SetPoint setpoint, Tango::DevDouble upper_threshold)
{
	SetPointConf conf = get_setpoint_conf(setpoint);

	conf.upper_threshold = upper_threshold;

	set_setpoint_conf(setpoint, conf);
}

//--------------------------------------------------------
// Tpg300SPManager::get_source()
//--------------------------------------------------------
Tango::DevShort Tpg300SPManager::get_source(SetPoint setpoint)
{
	SetPointConf conf = get_setpoint_conf(setpoint);

	return conf.source;
}

//--------------------------------------------------------
// Tpg300SPManager::set_source()
//--------------------------------------------------------
void Tpg300SPManager::set_source(SetPoint setpoint, Tango::DevShort source)
{
	SetPointConf conf = get_setpoint_conf(setpoint);

	conf.source = source;

	set_setpoint_conf(setpoint, conf);
}

//========================================================
//	Utilities methods
//========================================================

//--------------------------------------------------------
// Tpg300SPManager::get_setpoint_conf()
//--------------------------------------------------------
Tpg300SPManager::SetPointConf Tpg300SPManager::get_setpoint_conf(SetPoint setpoint)
{
	stringstream command_str;
	command_str << "SP" << char(setpoint);

	string result = proxy->write_read(command_str.str());

	size_t point_1_pos = result.find_first_of(POINT);
	size_t point_2_pos = result.find_last_of(POINT);

	if(point_1_pos == string::npos || point_2_pos == string::npos)
		Tango::Except::throw_exception("", "Parsing error", "get_setpoint_conf");

	SetPointConf conf;

	conf.lower_threshold = strtod(result.substr(0, point_1_pos).c_str(), 0);

	size_t sub_str_len = point_2_pos - point_1_pos - 1;
	conf.upper_threshold = strtod(result.substr(point_1_pos+1, sub_str_len).c_str(), 0);

	conf.source = atoi(result.substr(point_2_pos+1).c_str());

	return conf;
}

//--------------------------------------------------------
// Tpg300SPManager::set_setpoint_conf()
//--------------------------------------------------------
void Tpg300SPManager::set_setpoint_conf(SetPoint setpoint, Tpg300SPManager::SetPointConf conf)
{
	int l_exp = 0;
	double l_thres = convert(conf.lower_threshold, l_exp);

	int u_exp = 0;
	double u_thres = convert(conf.upper_threshold, u_exp);

	stringstream command_str;
	command_str.precision(1);

	command_str << "SP" << char(setpoint) << ","
			<< fixed << l_thres << "E" << l_exp << ","
			<< fixed << u_thres << "E" << u_exp << ","
			<< dec << conf.source << endl;

	proxy->write_read(command_str.str());
}

//--------------------------------------------------------
// Tpg300SPManager::convert()
//--------------------------------------------------------
double Tpg300SPManager::convert(double value, int& n)
{
    n = floor( log10(value) );

    return value * pow(10.0, -n);
}

}
