/*
 * Tpg300Thread.cpp
 *
 *  Created on: May 22, 2012
 *      Author: mdm
 */

#include <Tpg300Thread.h>
#include <Tpg300Channel.h>
#include <Tpg300SPManager.h>
#include <Tpg300Task.h>

namespace Tpg300_ns
{

//--------------------------------------------------------
// Tpg300Thread::Tpg300Thread()
//--------------------------------------------------------
Tpg300Thread::Tpg300Thread(std::vector<Tpg300Task *> task,
		Tango::DevULong msec, omni_condition* stop):
		task_vector(task), stop_condition(stop)
{
	abort = false;

	sleep_sec = msec/1000;

	sleep_nsec = (msec%1000)*1000000;

	condition_mutex = new omni_mutex();

	condition_var = new omni_condition(condition_mutex);

	start();
}

//--------------------------------------------------------
// Tpg300Thread::kill_thread()
//--------------------------------------------------------
void Tpg300Thread::kill_thread()
{
	abort = true;

	condition_var->signal();
}

//--------------------------------------------------------
// Tpg300Thread::~Tpg300Thread()
//--------------------------------------------------------
Tpg300Thread::~Tpg300Thread()
{
	delete condition_mutex;

	delete condition_var;
}

//--------------------------------------------------------
// Tpg300Thread::run()
//--------------------------------------------------------
void Tpg300Thread::run(void*)
{
	vector<Tpg300Task *>::const_iterator it = task_vector.begin();

	while(!abort)
	{
		(*it)->update();

		if(++it == task_vector.end())
			it=task_vector.begin();

		sleep();
	}

	stop_condition->signal();
}

//--------------------------------------------------------
// Tpg300Thread::sleep()
//--------------------------------------------------------
void Tpg300Thread::sleep()
{
	unsigned long sec;
	unsigned long nsec;

	omni_thread::get_time(&sec, &nsec, sleep_sec, sleep_nsec);

	condition_var->timedwait(sec, nsec);
}

}
